@@ -498,7 +498,7 @@ e_gw_connection_get_items (EGwConnection *cnc, const char *container, const char
         SoupSoapResponse *response;
         EGwConnectionStatus status;
         SoupSoapParameter *param, *subparam;
-
+	
         g_return_val_if_fail (E_IS_GW_CONNECTION (cnc), E_GW_CONNECTION_STATUS_INVALID_OBJECT);
 	
 	/* build the SOAP message */
@@ -511,9 +511,9 @@ e_gw_connection_get_items (EGwConnection *cnc, const char *container, const char
 	e_gw_message_write_string_parameter (msg, "container", NULL, container);
 	if (view)
 		e_gw_message_write_string_parameter (msg, "view", NULL, view);
+      	
 	if (filter) 
 		e_gw_filter_append_to_soap_message (filter, msg);
-	
 	e_gw_message_write_footer (msg);
 
         /* send message to server */
@@ -544,7 +544,76 @@ e_gw_connection_get_items (EGwConnection *cnc, const char *container, const char
 	     subparam = soup_soap_parameter_get_next_child_by_name (subparam, "item")) {
 		EGwItem *item;
 
-		item = e_gw_item_new_from_soap_parameter (container, subparam);
+		item = e_gw_item_new_from_soap_parameter (cnc->priv->user_email, container, subparam);
+		if (item)
+			*list = g_list_append (*list, item);
+        }
+               
+	/* free memory */
+        g_object_unref (response);
+	g_object_unref (msg);
+
+        return E_GW_CONNECTION_STATUS_OK;
+}
+
+EGwConnectionStatus
+e_gw_connection_get_items_from_ids (EGwConnection *cnc, const char *container, const char *view, GPtrArray *item_ids, GList **list)
+{
+        SoupSoapMessage *msg;
+        SoupSoapResponse *response;
+        EGwConnectionStatus status;
+        SoupSoapParameter *param, *subparam;
+	int i;
+        g_return_val_if_fail (E_IS_GW_CONNECTION (cnc), E_GW_CONNECTION_STATUS_INVALID_OBJECT);
+	
+	/* build the SOAP message */
+        msg = e_gw_message_new_with_header (cnc->priv->uri, cnc->priv->session_id, "getItemsRequest");
+        if (!msg) {
+                g_warning (G_STRLOC ": Could not build SOAP message");
+                return E_GW_CONNECTION_STATUS_UNKNOWN;
+        }
+
+	e_gw_message_write_string_parameter (msg, "container", NULL, container);
+	if (view)
+		e_gw_message_write_string_parameter (msg, "view", NULL, view);
+      	soup_soap_message_start_element (msg, "items", NULL, NULL);
+	for (i = 0; i < item_ids->len; i ++) {
+		char *id = g_ptr_array_index (item_ids, i);
+		e_gw_message_write_string_parameter (msg, "item", NULL, id);
+	}
+	soup_soap_message_end_element (msg);
+
+	e_gw_message_write_footer (msg);
+
+        /* send message to server */
+        response = e_gw_connection_send_message (cnc, msg);
+        if (!response) {
+                g_object_unref (msg);
+                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+        }
+
+        status = e_gw_connection_parse_response_status (response);
+        if (status != E_GW_CONNECTION_STATUS_OK) {
+		g_object_unref (response);
+                g_object_unref (msg);
+		return status;
+	}
+
+	/* if status is OK - parse result. return the list */	
+	param = soup_soap_response_get_first_parameter_by_name (response, "items");
+        if (!param) {
+                g_object_unref (response);
+                g_object_unref (msg);
+                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+        }
+	
+        /* parse these parameters into ecalcomponents*/
+        for (subparam = soup_soap_parameter_get_first_child_by_name (param, "item");
+	     subparam != NULL;
+	     subparam = soup_soap_parameter_get_next_child_by_name (subparam, "item")) {
+		EGwItem *item;
+
+		item = e_gw_item_new_from_soap_parameter (cnc->priv->user_email, container, subparam);
 		if (item)
 			*list = g_list_append (*list, item);
         }
@@ -570,7 +639,7 @@ e_gw_connection_get_deltas ( EGwConnection *cnc, GSList **adds, GSList **deletes
          msg = e_gw_message_new_with_header (cnc->priv->uri, cnc->priv->session_id, "getDeltaRequest"); 
          if (!msg) { 
                  g_warning (G_STRLOC ": Could not build SOAP message"); 
-		 g_object_unref (cnc); 
+		 // g_object_unref (cnc); 
                  return E_GW_CONNECTION_STATUS_UNKNOWN; 
          } 
         
@@ -583,7 +652,7 @@ e_gw_connection_get_deltas ( EGwConnection *cnc, GSList **adds, GSList **deletes
          response = e_gw_connection_send_message (cnc, msg); 
          if (!response) { 
                  g_object_unref (msg); 
-		 g_object_unref (cnc); 
+		 // g_object_unref (cnc); 
                  return E_GW_CONNECTION_STATUS_INVALID_RESPONSE; 
          } 
 
@@ -591,7 +660,7 @@ e_gw_connection_get_deltas ( EGwConnection *cnc, GSList **adds, GSList **deletes
          if (status != E_GW_CONNECTION_STATUS_OK) { 
  		g_object_unref (response); 
 		g_object_unref (msg); 
- 		g_object_unref (cnc); 
+		//	g_object_unref (cnc); 
  		return status; 
  	} 
 
@@ -600,13 +669,13 @@ e_gw_connection_get_deltas ( EGwConnection *cnc, GSList **adds, GSList **deletes
          if (!param) { 
                  g_object_unref (response); 
                  g_object_unref (msg); 
-		 g_object_unref (cnc); 
+		 // g_object_unref (cnc); 
                  return E_GW_CONNECTION_STATUS_INVALID_RESPONSE; 
          } 
 	
          if (!g_ascii_strcasecmp ( soup_soap_parameter_get_string_value (param), "0")) { 
                  g_message ("No deltas"); 
-		 g_object_unref (cnc); 
+		 // g_object_unref (cnc); 
                  return E_GW_CONNECTION_STATUS_OK; 
          }                 
 
@@ -614,8 +683,11 @@ e_gw_connection_get_deltas ( EGwConnection *cnc, GSList **adds, GSList **deletes
          if (!param) { 
                  g_object_unref (response); 
                  g_object_unref (msg); 
-		 g_object_unref (cnc); 
-                 return E_GW_CONNECTION_STATUS_INVALID_RESPONSE; 
+		 // g_object_unref (cnc); 
+//                 return E_GW_CONNECTION_STATUS_INVALID_RESPONSE; 
+		/* getting around the server behavior that deltas can be null
+		 * though changes is true */
+		 return E_GW_CONNECTION_STATUS_OK;
          } 
         
          /* process all deletes first*/ 
@@ -632,7 +704,7 @@ e_gw_connection_get_deltas ( EGwConnection *cnc, GSList **adds, GSList **deletes
                                  if (!param_id) { 
                                          g_object_unref (response); 
                                          g_object_unref (msg); 
-                                         g_object_unref (cnc); 
+					 // g_object_unref (cnc); 
                                  } 
                                  uid = (char *)soup_soap_parameter_get_string_value (param_id); 
                                  /*if (!e_cal_backend_cache_remove_component (cache, uid, NULL)) 
@@ -650,11 +722,11 @@ e_gw_connection_get_deltas ( EGwConnection *cnc, GSList **adds, GSList **deletes
 				/*process each item */  
 				EGwItem *item;
 				/*FIXME  pass the container id */
-				item = e_gw_item_new_from_soap_parameter ("Calendar", subparam);
+				item = e_gw_item_new_from_soap_parameter (cnc->priv->user_email, "Calendar", subparam);
                                 if (!item) { 
                                          g_object_unref (response); 
                                          g_object_unref (msg); 
-                                         g_object_unref (cnc); 
+					 // g_object_unref (cnc); 
  					return E_GW_CONNECTION_STATUS_INVALID_RESPONSE; 
                                  } 
                                  /*if (!e_cal_backend_cache_put_component (cache, comp)) 
@@ -672,7 +744,7 @@ e_gw_connection_get_deltas ( EGwConnection *cnc, GSList **adds, GSList **deletes
 			 EGwItem *item;
 			 /*process each item */
 			 /*item = get_item_from_updates (subparam);*/
-			 item = e_gw_item_new_from_soap_parameter ("Calendar", subparam);
+			 item = e_gw_item_new_from_soap_parameter (cnc->priv->user_email, "Calendar", subparam);
 			 if (item)
 				 *updates = g_slist_append (*updates, item);
                  } 
@@ -686,7 +758,7 @@ e_gw_connection_get_deltas ( EGwConnection *cnc, GSList **adds, GSList **deletes
 }
 
 EGwConnectionStatus
-e_gw_connection_send_item (EGwConnection *cnc, EGwItem *item, char **id)
+e_gw_connection_send_item (EGwConnection *cnc, EGwItem *item, GSList **id_list)
 {
 	SoupSoapMessage *msg;
 	SoupSoapResponse *response;
@@ -695,8 +767,8 @@ e_gw_connection_send_item (EGwConnection *cnc, EGwItem *item, char **id)
 	g_return_val_if_fail (E_IS_GW_CONNECTION (cnc), E_GW_CONNECTION_STATUS_INVALID_CONNECTION);
 	g_return_val_if_fail (E_IS_GW_ITEM (item), E_GW_CONNECTION_STATUS_INVALID_OBJECT);
 
-	if (id)
-		*id = NULL;
+	if (id_list)
+		*id_list = NULL;
 
 	/* compose SOAP message */
 	msg = e_gw_message_new_with_header (cnc->priv->uri, cnc->priv->session_id, "sendItemRequest");
@@ -721,13 +793,16 @@ e_gw_connection_send_item (EGwConnection *cnc, EGwItem *item, char **id)
 	}
 
 	status = e_gw_connection_parse_response_status (response);
-	if (status == E_GW_CONNECTION_STATUS_OK && id != NULL) {
+	if (status == E_GW_CONNECTION_STATUS_OK && id_list != NULL) {
 		SoupSoapParameter *param;
 
 		/* get the generated ID from the SOAP response */
-		param = soup_soap_response_get_first_parameter_by_name (response, "id");
-		if (param)
-			*id = soup_soap_parameter_get_string_value (param);
+		// for loop here to populate the list_ids.
+		for (param = soup_soap_response_get_first_parameter_by_name (response, "id");
+			param; param = soup_soap_response_get_next_parameter_by_name (response, param, "id")) {
+			
+			*id_list = g_slist_append (*id_list, soup_soap_parameter_get_string_value (param));
+		}
 	}
 
 	g_object_unref (msg);
@@ -866,7 +941,7 @@ e_gw_connection_get_item (EGwConnection *cnc, const char *container, const char
                 return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
         }
 	
-       	*item = e_gw_item_new_from_soap_parameter (container, param);
+       	*item = e_gw_item_new_from_soap_parameter (cnc->priv->user_email, container, param);
 	
                
 	/* free memory */
@@ -914,6 +989,144 @@ e_gw_connection_remove_item (EGwConnection *cnc, const char *container, const ch
 	return status;
 }
 
+EGwConnectionStatus
+e_gw_connection_remove_items (EGwConnection *cnc, const char *container, GList *item_ids)
+{
+	SoupSoapMessage *msg;
+	SoupSoapResponse *response;
+	EGwConnectionStatus status;
+
+	g_return_val_if_fail (E_IS_GW_CONNECTION (cnc), E_GW_CONNECTION_STATUS_INVALID_CONNECTION);
+
+	/* build the SOAP message */
+	msg = e_gw_message_new_with_header (cnc->priv->uri, cnc->priv->session_id, "removeItemsRequest");
+	if (container && *container)
+		e_gw_message_write_string_parameter (msg, "container", NULL, container);
+
+	soup_soap_message_start_element (msg, "items", NULL, NULL);
+	for (; item_ids != NULL; item_ids = g_list_next (item_ids))
+		e_gw_message_write_string_parameter (msg, "item", NULL, item_ids->data);
+	soup_soap_message_end_element (msg);
+	e_gw_message_write_footer (msg);
+
+	/* send message to server */
+	response = e_gw_connection_send_message (cnc, msg);
+	if (!response) {
+		g_object_unref (msg);
+		return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+	}
+
+	status = e_gw_connection_parse_response_status (response);
+
+	/* free memory */
+	g_object_unref (response);
+	g_object_unref (msg);
+
+	return status;
+}
+
+EGwConnectionStatus
+e_gw_connection_accept_request (EGwConnection *cnc, const char *id, const char *accept_level)
+{
+	SoupSoapMessage *msg;
+	int status;
+	SoupSoapResponse *response;
+
+	msg = e_gw_message_new_with_header (cnc->priv->uri, cnc->priv->session_id, "acceptRequest");
+	soup_soap_message_start_element (msg, "items", NULL, NULL);
+	e_gw_message_write_string_parameter (msg, "item", NULL, id);
+	soup_soap_message_end_element (msg);
+	e_gw_message_write_string_parameter (msg, "acceptLevel", NULL, accept_level);
+	e_gw_message_write_footer (msg);
+
+	response = e_gw_connection_send_message (cnc, msg);
+        if (!response) {
+                g_object_unref (msg);
+                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+        }
+
+        status = e_gw_connection_parse_response_status (response);
+	g_object_unref (response);
+	g_object_unref (msg);
+	return status;
+}
+
+EGwConnectionStatus
+e_gw_connection_decline_request (EGwConnection *cnc, const char *id)
+{
+	SoupSoapMessage *msg;
+	int status;
+	SoupSoapResponse *response;
+
+	msg = e_gw_message_new_with_header (cnc->priv->uri, cnc->priv->session_id, "declineRequest");
+	soup_soap_message_start_element (msg, "items", NULL, NULL);
+	e_gw_message_write_string_parameter (msg, "item", NULL, id);
+	soup_soap_message_end_element (msg);
+	e_gw_message_write_footer (msg);
+
+	response = e_gw_connection_send_message (cnc, msg);
+        if (!response) {
+                g_object_unref (msg);
+                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+        }
+
+        status = e_gw_connection_parse_response_status (response);
+	g_object_unref (response);
+	g_object_unref (msg);
+	return status;
+}
+
+EGwConnectionStatus
+e_gw_connection_retract_request (EGwConnection *cnc, const char *id, const char *comment, gboolean retract_all, gboolean resend)
+{
+	SoupSoapMessage *msg;
+	int status;
+	SoupSoapResponse *response;
+
+	msg = e_gw_message_new_with_header (cnc->priv->uri, cnc->priv->session_id, "retractRequest");
+	soup_soap_message_start_element (msg, "items", NULL, NULL);
+	e_gw_message_write_string_parameter (msg, "item", NULL, id);
+	soup_soap_message_end_element (msg);
+	/* comment, FALSE, FALSE to be filled in later. */
+	e_gw_message_write_footer (msg);
+
+	response = e_gw_connection_send_message (cnc, msg);
+        if (!response) {
+                g_object_unref (msg);
+                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+        }
+
+        status = e_gw_connection_parse_response_status (response);
+	g_object_unref (response);
+	g_object_unref (msg);
+	return status;
+}
+
+EGwConnectionStatus
+e_gw_connection_complete_request (EGwConnection *cnc, const char *id)
+{
+	SoupSoapMessage *msg;
+	int status;
+	SoupSoapResponse *response;
+
+	msg = e_gw_message_new_with_header (cnc->priv->uri, cnc->priv->session_id, "completeRequest");
+	soup_soap_message_start_element (msg, "items", NULL, NULL);
+	e_gw_message_write_string_parameter (msg, "item", NULL, id);
+	soup_soap_message_end_element (msg);
+	e_gw_message_write_footer (msg);
+
+	response = e_gw_connection_send_message (cnc, msg);
+        if (!response) {
+                g_object_unref (msg);
+                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+        }
+
+        status = e_gw_connection_parse_response_status (response);
+	g_object_unref (response);
+	g_object_unref (msg);
+	return status;
+}
+
 const char *
 e_gw_connection_get_uri (EGwConnection *cnc)
 {
